@@ -233,17 +233,19 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
     Adapter class for Twisted WebSocket client and server protocols.
     """
 
-    peer = '<never connected>'
-
     log = txaio.make_logger()
 
+    peer = None
+    peer_transport = None
+
     def connectionMade(self):
         # the peer we are connected to
         try:
             self.peer = peer2str(self.transport.getPeer())
-        except AttributeError:
+        except (AttributeError, NotImplementedError):
             # ProcessProtocols lack getPeer()
             self.peer = 'process:{}'.format(self.transport.pid)
+        self.peer_transport = 'websocket'
 
         self._connectionMade()
         self.log.debug('Connection made to {peer}', peer=self.peer)
@@ -355,7 +357,7 @@ class WebSocketServerProtocol(WebSocketAdapterProtocol, protocol.WebSocketServer
 
     log = txaio.make_logger()
 
-    def get_channel_id(self, channel_id_type='tls-unique'):
+    def get_channel_id(self, channel_id_type=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
         """
@@ -379,7 +381,7 @@ class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClient
         self.log.debug("Starting TLS upgrade")
         self.transport.startTLS(self.factory.contextFactory)
 
-    def get_channel_id(self, channel_id_type='tls-unique'):
+    def get_channel_id(self, channel_id_type=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
         """
@@ -856,3 +858,6 @@ class WampWebSocketClientFactory(websocket.WampWebSocketClientFactory, WebSocket
         kwargs['protocols'] = self._protocols
 
         WebSocketClientFactory.__init__(self, *args, **kwargs)
+
+        # Reduce the factory logs noise
+        self.noisy = False
