@@ -33,11 +33,14 @@ import base64
 import math
 import random
 import binascii
+import socket
+import subprocess
+
+from typing import Optional
 from datetime import datetime, timedelta
 from pprint import pformat
 from array import array
 
-
 import txaio
 
 try:
@@ -64,7 +67,8 @@ __all__ = ("public",
            "generate_token",
            "generate_activation_code",
            "generate_serial_number",
-           "generate_user_password")
+           "generate_user_password",
+           "machine_id")
 
 
 def public(obj):
@@ -133,17 +137,14 @@ def encode_truncate(text, limit, encoding='utf8', return_encoded=True):
 
 
 @public
-def xor(d1, d2):
+def xor(d1: bytes, d2: bytes) -> bytes:
     """
     XOR two binary strings of arbitrary (equal) length.
 
     :param d1: The first binary string.
-    :type d1: binary
     :param d2: The second binary string.
-    :type d2: binary
 
     :returns: XOR of the binary strings (``XOR(d1, d2)``)
-    :rtype: bytes
     """
     if type(d1) != bytes:
         raise Exception("invalid type {} for d1 - must be binary".format(type(d1)))
@@ -169,6 +170,11 @@ def utcstr(ts=None):
     Note: to parse an ISO 8601 formatted string, use the **iso8601**
     module instead (e.g. ``iso8601.parse_date("2014-05-23T13:03:44.123Z")``).
 
+    >>> txaio.time_ns()
+    1641121311914026419
+    >>> int(iso8601.parse_date(utcnow()).timestamp() * 1000000000.)
+    1641121313209000192
+
     :param ts: The timestamp to format.
     :type ts: instance of :py:class:`datetime.datetime` or ``None``
 
@@ -350,7 +356,11 @@ Delta" and telephone operators' "Is that 'd' as in 'dog'?".
 
 
 @public
-def generate_token(char_groups, chars_per_group, chars=None, sep=None, lower_case=False):
+def generate_token(char_groups: int,
+                   chars_per_group: int,
+                   chars: Optional[str] = None,
+                   sep: Optional[str] = None,
+                   lower_case: Optional[bool] = False) -> str:
     """
     Generate cryptographically strong tokens, which are strings like `M6X5-YO5W-T5IK`.
     These can be used e.g. for used-only-once activation tokens or the like.
@@ -380,27 +390,17 @@ def generate_token(char_groups, chars_per_group, chars=None, sep=None, lower_cas
     * token(6): ``NXW9-74LU-6NUH-VLPV-X6AG-QUE3``
 
     :param char_groups: Number of character groups (or characters if chars_per_group == 1).
-    :type char_groups: int
-
     :param chars_per_group: Number of characters per character group (or 1 to return a token with no grouping).
-    :type chars_per_group: int
-
     :param chars: Characters to choose from. Default is 27 character subset
         of the ISO basic Latin alphabet (see: ``DEFAULT_TOKEN_CHARS``).
-    :type chars: str or None
-
     :param sep: When separating groups in the token, the separater string.
-    :type sep: str
-
     :param lower_case: If ``True``, generate token in lower-case.
-    :type lower_case: bool
 
     :returns: The generated token.
-    :rtype: str
     """
     assert(type(char_groups) == int)
     assert(type(chars_per_group) == int)
-    assert(chars is None or type(chars) == str)
+    assert(chars is None or type(chars) == str), 'chars must be str, was {}'.format(type(chars))
     chars = chars or DEFAULT_TOKEN_CHARS
     if lower_case:
         chars = chars.lower()
@@ -425,6 +425,25 @@ def generate_activation_code():
     return generate_token(char_groups=3, chars_per_group=4, chars=DEFAULT_TOKEN_CHARS, sep='-', lower_case=False)
 
 
+_PAT_ACTIVATION_CODE = re.compile('^([' + DEFAULT_TOKEN_CHARS + ']{4,4})-([' + DEFAULT_TOKEN_CHARS + ']{4,4})-([' + DEFAULT_TOKEN_CHARS + ']{4,4})$')
+
+
+@public
+def parse_activation_code(code: str):
+    """
+    Parse an activation code generated by :func:<autobahn.util.generate_activation_code>:
+
+    .. code:: console
+
+        "RWCN-94NV-CEHR" -> ("RWCN", "94NV", "CEHR") | None
+
+    :param code: The code to parse, e.g. ``'W97F-96MJ-YGJL'``.
+    :return: If the string is a properly conforming activation code, return
+        the matched pattern, otherwise return ``None``.
+    """
+    return _PAT_ACTIVATION_CODE.match(code)
+
+
 @public
 def generate_user_password():
     """
@@ -797,7 +816,10 @@ class ObservableMixin(object):
                 if handler is None:
                     del self._listeners[event]
                 else:
-                    self._listeners[event].discard(handler)
+                    try:
+                        self._listeners[event].remove(handler)
+                    except ValueError:
+                        pass
 
     def fire(self, event, *args, **kwargs):
         """
@@ -866,3 +888,27 @@ def _maybe_tls_reason(instance):
             msg=ssl_error[2],
         )
     return ""
+
+
+def machine_id() -> str:
+    """
+    For informational purposes, get a unique ID or serial for this machine (device).
+
+    :returns: Unique machine (device) ID (serial), e.g. ``81655b901e334fc1ad59cbf2719806b7``.
+    """
+    from twisted.python.runtime import platform
+
+    if platform.isLinux():
+        try:
+            # why this? see: http://0pointer.de/blog/projects/ids.html
+            with open('/var/lib/dbus/machine-id', 'r') as f:
+                return f.read().strip()
+        except:
+            # Non-dbus using Linux, get a hostname
+            return socket.gethostname()
+    elif platform.isMacOSX():
+        import plistlib
+        plist_data = subprocess.check_output(["ioreg", "-rd1", "-c", "IOPlatformExpertDevice", "-a"])
+        return plistlib.loads(plist_data)[0]["IOPlatformSerialNumber"]
+    else:
+        return socket.gethostname()
