@@ -37,7 +37,7 @@ from autobahn.wamp import types
 from autobahn.wamp import role
 from autobahn.wamp import exception
 from autobahn.wamp.exception import ApplicationError, ProtocolError, SessionNotReady, SerializationError, TypeCheckError
-from autobahn.wamp.interfaces import IPayloadCodec, IAuthenticator  # noqa
+from autobahn.wamp.interfaces import ISession, IPayloadCodec, IAuthenticator  # noqa
 from autobahn.wamp.types import SessionDetails, CloseDetails, EncodedPayload
 from autobahn.exception import PayloadExceededError
 from autobahn.wamp.request import \
@@ -730,7 +730,7 @@ class ApplicationSession(BaseSession):
                         invoke_kwargs = msg.kwargs if msg.kwargs else dict()
 
                         if handler.details_arg:
-                            invoke_kwargs[handler.details_arg] = types.EventDetails(subscription, msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, retained=msg.retained, enc_algo=msg.enc_algo, forward_for=msg.forward_for)
+                            invoke_kwargs[handler.details_arg] = types.EventDetails(subscription, msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, transaction_hash=msg.transaction_hash, retained=msg.retained, enc_algo=msg.enc_algo, forward_for=msg.forward_for)
 
                         # FIXME: https://github.com/crossbario/autobahn-python/issues/764
                         def _success(_):
@@ -1020,6 +1020,7 @@ class ApplicationSession(BaseSession):
                                                                                         caller_authid=msg.caller_authid,
                                                                                         caller_authrole=msg.caller_authrole,
                                                                                         procedure=proc,
+                                                                                        transaction_hash=msg.transaction_hash,
                                                                                         enc_algo=msg.enc_algo)
 
                             on_reply = txaio.as_future(endpoint.fn, *invoke_args, **invoke_kwargs)
@@ -1289,13 +1290,13 @@ class ApplicationSession(BaseSession):
     @public
     def onJoin(self, details):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISession.onJoin`
+        Implements :meth:`autobahn.wamp.interfaces.ISession.onJoin`
         """
 
     @public
     def onWelcome(self, msg):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISession.onWelcome`
+        Implements :meth:`autobahn.wamp.interfaces.ISession.onWelcome`
         """
 
     def _errback_outstanding_requests(self, exc):
@@ -1339,7 +1340,7 @@ class ApplicationSession(BaseSession):
     @public
     def onLeave(self, details):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISession.onLeave`
+        Implements :meth:`autobahn.wamp.interfaces.ISession.onLeave`
         """
         if details.reason != CloseDetails.REASON_DEFAULT:
             self.log.warn('session closed with reason {reason} [{message}]', reason=details.reason, message=details.message)
@@ -1357,7 +1358,7 @@ class ApplicationSession(BaseSession):
     @public
     def leave(self, reason=None, message=None):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISession.leave`
+        Implements :meth:`autobahn.wamp.interfaces.ISession.leave`
         """
         if not self._session_id:
             raise SessionNotReady("session hasn't joined a realm")
@@ -1378,7 +1379,7 @@ class ApplicationSession(BaseSession):
     @public
     def onDisconnect(self):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISession.onDisconnect`
+        Implements :meth:`autobahn.wamp.interfaces.ISession.onDisconnect`
         """
         # fire TransportLost on any _still_ outstanding requests
         # (these should have been already cleaned up in onLeave() - when
@@ -1389,7 +1390,7 @@ class ApplicationSession(BaseSession):
     @public
     def publish(self, topic, *args, **kwargs):
         """
-        Implements :func:`autobahn.wamp.interfaces.IPublisher.publish`
+        Implements :meth:`autobahn.wamp.interfaces.IPublisher.publish`
         """
         assert(type(topic) == str)
         assert(args is None or type(args) in (list, tuple))
@@ -1480,7 +1481,7 @@ class ApplicationSession(BaseSession):
     @public
     def subscribe(self, handler, topic=None, options=None, check_types=False):
         """
-        Implements :func:`autobahn.wamp.interfaces.ISubscriber.subscribe`
+        Implements :meth:`autobahn.wamp.interfaces.ISubscriber.subscribe`
         """
         assert((callable(handler) and topic is not None) or (hasattr(handler, '__class__') and not check_types))
         assert(topic is None or type(topic) == str)
@@ -1583,7 +1584,7 @@ class ApplicationSession(BaseSession):
     @public
     def call(self, procedure, *args, **kwargs):
         """
-        Implements :func:`autobahn.wamp.interfaces.ICaller.call`
+        Implements :meth:`autobahn.wamp.interfaces.ICaller.call`
         """
         assert(type(procedure) == str)
         assert(args is None or type(args) in (list, tuple))
@@ -1681,7 +1682,7 @@ class ApplicationSession(BaseSession):
     @public
     def register(self, endpoint, procedure=None, options=None, prefix=None, check_types=False):
         """
-        Implements :func:`autobahn.wamp.interfaces.ICallee.register`
+        Implements :meth:`autobahn.wamp.interfaces.ICallee.register`
         """
         assert((callable(endpoint) and procedure is not None) or (hasattr(endpoint, '__class__') and not check_types))
         assert(procedure is None or type(procedure) == str)
@@ -1939,7 +1940,7 @@ class _SessionShim(ApplicationSession):
 
 
 # ISession.register collides with the abc.ABCMeta.register method
-# ISession.register(ApplicationSession)
+ISession._abc_register(ApplicationSession)
 
 
 class ApplicationSessionFactory(object):
